## This file auto-generated by InlineX-C2XS-0.26

use strict;
use warnings;

print "1..2\n";

eval{require List::Uniqnum;};
if($@) {
  warn "\$\@: $@";
  print "not ok 1\n";
}
else {print "ok 1\n"}

if($List::Uniqnum::VERSION eq '0.08') { print "ok 2\n" }
else {
  warn "\nVersion is $List::Uniqnum::VERSION\n";
  print "not ok 2\n";
}
